# NOTE: Derived from blib/lib/Net/EasyTCP.pm.
# Changes made here will be lost when autosplit again.
# See AutoSplit.pm.
package Net::EasyTCP;

#line 2419 "blib/lib/Net/EasyTCP.pm (autosplit into blib/lib/auto/Net/EasyTCP/do_one_loop.al)"
#
# This method does "one loop" of server work and returns ASAP
# It should be called very frequently, either through a while() loop in the program
# or through the start() method
#
# It accepts new clients, accepts data from them, and fires off any callback events as necessary
#
sub do_one_loop {
	my $self = shift;
	my @ready;
	my $clientsock;
	my $tempdata;
	my $serverclient;
	my $realdata;
	my $result;
	my $negotiatingtimeout = 45;
	my $peername;
	my $remoteport;
	my $remoteip;

	if ($self->{_mode} ne "server") {
		$@ = "$self->{_mode} cannot use method do_one_loop()";
		return undef;
	}
	$self->{_lastglobalkeygentime} ||= time;
	@ready = $self->{_selector}->can_read(1);
	foreach (@ready) {
		if ($_ == $self->{_sock}) {

			#
			# The SERVER SOCKET is ready for accepting a new client
			#
			$clientsock = $self->{_sock}->accept();
			if (!$clientsock) {
				$@ = "Error while accepting new connection: $!";
				return undef;
			}

			#
			# We get remote IP and port, we'll need them to see if client is allowed or not
			#
			$peername = getpeername($clientsock) or next;
			($remoteport, $remoteip) = sockaddr_in($peername) or next;
			$remoteip = inet_ntoa($remoteip) or next;

			#
			# We create a new client object and
			# We see if client is allowed to connect to us
			#
			if (scalar(keys %{ $self->{_clientip} }) && !$self->{_clientip}{$remoteip}) {

				#
				# Client's IP is not allowed to connect to us
				#
				close($clientsock);
			}
			else {

				#
				# We add it to our SELECTOR pool :
				#
				$self->{_selector}->add($clientsock);

				#
				# We create a new client object:
				#
				$self->{_clients}->{$clientsock} = _new_client(
					$self,
					"_sock"       => $clientsock,
					"_remoteport" => $remoteport,
					"_remoteip"   => $remoteip
				);

				#
				# We initialize some client variables:
				#
				$self->{_clients}->{$clientsock}->{_serial}                 = ++$_SERIAL;
				$self->{_clients}->{$clientsock}->{_compatabilityscalar}    = _genrandstring(129);
				$self->{_clients}->{$clientsock}->{_compatabilityreference} = _gencompatabilityreference($self->{_clients}->{$clientsock}->{_compatabilityscalar});

				#
				# And we make it inherit some stuff from the server :
				#
				$self->{_clients}->{$clientsock}->{_donotencrypt}      = $self->{_donotencrypt};
				$self->{_clients}->{$clientsock}->{_donotencryptwith}  = $self->{_donotencryptwith};
				$self->{_clients}->{$clientsock}->{_donotcompress}     = $self->{_donotcompress};
				$self->{_clients}->{$clientsock}->{_donotcompresswith} = $self->{_donotcompresswith};
				$self->{_clients}->{$clientsock}->{_password}          = $self->{_password};
				$self->{_clients}->{$clientsock}->{_callbacks}         = $self->{_callbacks};
				$self->{_clients}->{$clientsock}->{_welcome}           = $self->{_welcome};
				$self->{_clients}->{$clientsock}->{_selector}          = $self->{_selector};
			}
		}
		else {

			#
			# One of the CLIENT sockets are ready
			#
			$result = sysread($_, $tempdata, 4096);
			$serverclient = $self->{_clients}->{$_};
			if (!defined $result) {

				#
				# Error somewhere during reading from that client
				#
				_callback($serverclient, "disconnect");
				$serverclient->close();
				delete $self->{_clients}->{$_};
			}
			elsif ($result == 0) {

				#
				# Client closed connection
				#
				_callback($serverclient, "disconnect");
				$serverclient->close();
				delete $self->{_clients}->{$_};
			}
			else {

				#
				# Client sent us some good data (not necessarily a full packet)
				#
				$serverclient->{_databuffer} .= $tempdata;
				while (defined($realdata = _extractdata($serverclient))) {
					if (!$realdata) {

						# It's internal protocol data
						_parseinternaldata($serverclient);
					}
					else {

						# We found something and it's real data
						_callback($serverclient, "data");
					}
				}
			}
		}
	}

	#
	# Now we check on all the serverclients still negotiating and help them finish negotiating
	# or weed out the ones timing out
	#
	foreach (keys %{ $self->{_clients} }) {
		$serverclient = $self->{_clients}->{$_};
		if ($serverclient->{_negotiating}) {
			if (_serverclient_negotiate($serverclient)) {
				_callback($serverclient, "connect");
			}
			elsif ((time - $serverclient->{_negotiating}) > $negotiatingtimeout) {
				$serverclient->close();
				delete $self->{_clients}->{$_};
			}
		}
	}

	#
	# Now we re-generate the RSA keys if it's been over an hour
	#
	if (!$self->{_donotencrypt} && !$self->{_donotencryptwith}{"B"} && ((time - $self->{_lastglobalkeygentime}) >= 3600)) {
		if (!_generateglobalkeypair('Crypt::RSA')) {
			$@ = "Could not generate global Crypt::RSA keypairs. $@";
			return undef;
		}
		$self->{_lastglobalkeygentime} = time;
	}
	return 1;
}

# end of Net::EasyTCP::do_one_loop
1;
